/*!
* Module dependencies.
*/

var mongoose = require('mongoose');
var config = require('../../config/env/development');
var async = require('async');
var utils = require('../../utils');
var fs = require('fs');
var path = require('path');

var Character = mongoose.model('Character');

exports.read = function (req, res) {
    var query = {};
    var language = req.query.language;
    Character.find(query).sort({ id: -1 }).exec(async function (err, characters) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read characters data', JSON.stringify(err.errors));
        } else {
            if(language){
                let response=[], data;
                 for (let index = 0; index < characters.length; index++) {
                     data=await transalationService(characters[index],language);
                     response.push(data.character)
                     if (data.update)
                       udpate(characters[index]._id, data.character)
                 }
                 return utils.sendSuccessResponse(res, 200, response);
             }else{
                 return utils.sendSuccessResponse(res, 200, characters);
             }
        }
    })
}

exports.detail =  function (req, res) {
    var characterId = req.params.characterId;
    var language = req.query.language;
    var data;
    Character.findOne({ _id: characterId }).exec(async function (err, character) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read character data', JSON.stringify(err.errors));
        } else {
            if(language){
                data=await transalationService(character,language);
                if (data.update)
                   udpate(characterId, data.character)
            }
            return utils.sendSuccessResponse(res, 200,language? data.character:character);
        }
    })
}

exports.create = function (req, res) {
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    if (req.body.dialogues) {
        req.body.dialogues = JSON.parse(req.body.dialogues);
    }
    if (req.body.excludedUtilities) {
        req.body.excludedUtilities = JSON.parse(req.body.excludedUtilities);
    }
    var character = new Character(req.body);
    character.save(function (err, savedCharacter) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to create a new character', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, savedCharacter);
        }
    });
}

exports.update = function (req, res) {
    var characterId = req.params.characterId;
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    var updateParams = req.body;
    updateParams.translation={name:{},dialogues:{}};
    Character.findOneAndUpdate({ _id: characterId }, { $set: updateParams }, { new: true, upsert: true }).exec(function (err, updatedCharacter) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to update character', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, updatedCharacter);
        }
    })
}

exports.delete = function (req, res) {
    var characterId = req.params.characterId;
    Character.remove({ _id: characterId }).exec(function (err, data) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to delete character entry', JSON.stringify(err.errors));
        } else {
            if (data['result']['n'] == 1) {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "Deleted character",
                        success: true
                    }
                );
            } else {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "No character found to delete",
                        success: false
                    }
                );
            }
        }
    })
}

async function transalationService(character,language){
    var response={
        translation:{
            name:{[language]:{}},
            dialogues:{
                [language]:{
            generic: [],
            inShop: [],
            afterShop: [],
            frontDesk: [],
            nearDecor: []}}
    }};
   
    var update=false;
    if(!character.translation){
        character["translation"] =response.translation 
    }
    // !('translation' in character) && (character.translation =response.translation )  
    if (!character || !character.translation || !character.translation.name || !character.translation.name[language] || !character.translation.name[language].length) {
      update = true;
      response.translation.name[language] = character.name? await utils.translateText(character.name, language):"";
      character["translation"]["name"] =(character["translation"]&&character["translation"]["name"])? {
        ...character["translation"]["name"],
        ...response.translation.name
      }:response.translation.name
    }
    if (!character || !character.translation || !character.translation.dialogues || !character.translation.dialogues[language] || !character.translation.dialogues[language]) {
      update = true;
      var type=["generic","inShop","afterShop","frontDesk","nearDecor"]
      for (let i = 0; i < type.length; i++) {
        for (let index = 0; index < character.dialogues[type[i]].length; index++) {
            response.translation.dialogues[language][type[i]][index] = character.dialogues[type[i]][index]?await utils.translateText(character.dialogues[type[i]][index], language):"";
          }  
          
      }
      
      character["translation"]["dialogues"] =(character["translation"] && character["translation"]["dialogues"])? {
        ...character["translation"]["dialogues"],
        ...response.translation.dialogues
      }:response.translation.dialogues
    }

    return {character,update};
}

function udpate(id,data){
    Character.findOneAndUpdate({ _id: id }, { $set: data }, { new: false, upsert: false }).exec(function (err, updatedBuilding) {
        if (err) {
            console.log(err);
        }
    })
}