/*!
* Module dependencies.
*/

var mongoose = require('mongoose');
var config = require('../../config/env/development');
var async = require('async');
var utils = require('../../utils');
var fs = require('fs');
var path = require('path');

var UserFollower = mongoose.model('UserFollower');
var UserProfile = mongoose.model('UserProfile')


exports.followers = function (req, res) {
  var deviceId = req.deviceId
  async.waterfall([
    function (callback) {
      UserProfile.findOne({ deviceId: deviceId }).exec(callback)
    },
    function (userprofile, callback) {
      UserFollower.find({ followerId: userprofile._id }).populate({ path: 'userId', select: 'level coins xp' }).exec(callback)
    }
  ], function (err, result) {
    if (err) {
      return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to user profile', JSON.stringify(err.errors));
    } else {
      return utils.sendSuccessResponse(res, 200, result.map(function (following) {
        return following.userId
      }));
    }
  })
}

exports.following = function (req, res) {
  var deviceId = req.deviceId
  async.waterfall([
    function (callback) {
      UserProfile.findOne({ deviceId: deviceId }).exec(callback)
    },
    function (userprofile, callback) {
      UserFollower.find({ userId: userprofile._id }).populate({ path: 'followerId', select: 'level coins xp' }).exec(callback)
    }
  ], function (err, result) {
    if (err) {
      return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to user profile', JSON.stringify(err.errors));
    } else {
      return utils.sendSuccessResponse(res, 200, result.map(function (following) {
        return following.followerId
      }));
    }
  })
}

exports.follow = function (req, res) {
  var deviceId = req.deviceId
  var followerId = req.params.followerId
  console.log("REQUEST = ", req);
  console.log("DEVICE ID = ", deviceId, "  FOLLOWER ID = ", followerId);
  async.waterfall([
    function (callback) {
      UserProfile.findOne({ deviceId: deviceId }).exec(callback)
    },
    function (userprofile, callback) {
      UserFollower.findOneAndUpdate({ userId: userprofile._id, followerId: followerId }, { $set: { isFollowing: true } }, { new: true, upsert: true }).exec(callback)
    }], function (err, result) {
      if (err) {
        return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to follow user', JSON.stringify(err.errors));
      } else {
        return utils.sendSuccessResponse(res, 200, result);
      }
    }
  )
}

exports.unfollow = function (req, res) {
  var deviceId = req.deviceId
  var followerId = req.params.followerId
  async.waterfall([
    function (callback) {
      UserProfile.findOne({ deviceId: deviceId }).exec(callback)
    },
    function (userprofile, callback) {
      UserFollower.findOneAndUpdate({ userId: userprofile._id, followerId: followerId }, { $set: { isFollowing: false } }, { new: true }).exec(callback)
    }], function (err, result) {
      if (err) {
        return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to unfollow user', JSON.stringify(err.errors));
      } else {
        return utils.sendSuccessResponse(res, 200, result);
      }
    }
  )
}