/*!
* Module dependencies.
*/

var mongoose = require('mongoose');
var config = require('../../config/env/development');
var async = require('async');
var utils = require('../../utils');
var fs = require('fs');
var path = require('path');
var pushNotify = require('./pushnotify');
var moment = require('moment');

var Notification = mongoose.model('Notification');
var UserProfile = mongoose.model('UserProfile');

exports.read = function (req, res) {
    var query = {};
    if (req.query.enabled) {
        query['enabled'] = req.query.enabled;
    }
    if (req.query.type) {
        query['type'] = req.query.type;
    }
    Notification.find(query).sort({ id: -1 }).exec(function (err, notifications) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read notifications data', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, notifications);
        }
    })
}

exports.detail = function (req, res) {
    var notificationId = req.params.notificationId;
    Notification.findOne({ _id: notificationId }).exec(function (err, notification) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read notification data', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, notification);
        }
    })
}

exports.create = function (req, res) {
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    console.log("req.body", req.body);
    var notification = new Notification(req.body);
    notification.save(function (err, savedNotification) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to create a new notification', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, savedNotification);
        }
    });
}

exports.update = function (req, res) {
    var notificationId = req.params.notificationId;
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    var updateParams = req.body;
    Notification.findOneAndUpdate({ _id: notificationId }, { $set: updateParams }, { new: true, upsert: true }).exec(function (err, updatedNotification) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to update notification', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, updatedNotification);
        }
    })
}

exports.delete = function (req, res) {
    var notificationId = req.params.notificationId;
    Notification.remove({ _id: notificationId }).exec(function (err, data) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to delete notification entry', JSON.stringify(err.errors));
        } else {
            if (data['result']['n'] == 1) {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "Deleted notification",
                        success: true
                    }
                );
            } else {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "No notification found to delete",
                        success: false
                    }
                );
            }
        }
    })
}

exports.sentNotifications = function (req, res) {


}

exports.sendPendingNotifications = function (req, res) {
    var currentDate = new Date();
    async.series([
        function (callback) {
            Notification.find({ deliveryDate: { $lte: currentDate }, enabled: true }).exec(callback);
        }, function (callback) {
            UserProfile.find({}, { pushToken: 1 }).lean().exec(callback);
        }
    ], function (err, data) {
        var notificationEvents = data[0];
        var pushTokens = data[1].map((object) => {
            return object.pushToken;
        });
        //TODO: spilt to array of 1000 pushTokens
        async.eachSeries(notificationEvents, function (event, callback) {
            pushNotify.sendPushNotification(pushTokens, {
                message: event.data.message,
                type: event.type
            });
            if (!event.isRepeat) {
                event.enabled = false;
            } else {
                switch (event.frequency) {
                    case 'daily':
                        event.deliveryDate = moment(event.deliveryDate).add(1, "days").toISOString();
                        break;
                    case 'weekly':
                        event.deliveryDate = moment(event.deliveryDate).add(7, "days").toISOString();
                        break;
                    case 'monthly':
                        event.deliveryDate = moment(event.deliveryDate).add(1, "month").toISOString();
                        break;
                }
            }
            console.log(event);
            event.save(callback);
            // callback();
        }, function (err) {
            res.send("OK")
        });
    });
}
