/*!
* Module dependencies.
*/

var mongoose = require('mongoose');
var config = require('../../config/env/development');
var async = require('async');
var utils = require('../../utils');
var fs = require('fs');
var path = require('path');

var Product = mongoose.model('Product');

exports.read = function (req, res) {
    var query = {};
    if (req.query.enabled) {
        query['enabled'] = req.query.enabled;
    }
    if (req.query.type) {
        query['type'] = req.query.type;
    }
    Product.find(query).sort({ id: -1 }).exec(function (err, products) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read products data', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, products);
        }
    })
}

exports.detail = function (req, res) {
    var productId = req.params.productId;
    Product.findOne({ _id: productId }).exec(function (err, product) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to read product data', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, product);
        }
    })
}

exports.create = function (req, res) {
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    console.log("req.body", req.body);
    var product = new Product(req.body);
    product.save(function (err, savedProduct) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to create a new product', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, savedProduct);
        }
    });
}

exports.update = function (req, res) {
    var productId = req.params.productId;
    if (req.file) {
        req.body.imageUrl = config.assets.url + '/' + req.file.filename;
    }
    var updateParams = req.body;
    Product.findOneAndUpdate({ _id: productId }, { $set: updateParams }, { new: true, upsert: true }).exec(function (err, updatedProduct) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to update product', JSON.stringify(err.errors));
        } else {
            return utils.sendSuccessResponse(res, 200, updatedProduct);
        }
    })
}

exports.delete = function (req, res) {
    var productId = req.params.productId;
    Product.remove({ _id: productId }).exec(function (err, data) {
        if (err) {
            return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to delete product entry', JSON.stringify(err.errors));
        } else {
            if (data['result']['n'] == 1) {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "Deleted product",
                        success: true
                    }
                );
            } else {
                return utils.sendSuccessResponse(res, 200,
                    {
                        message: "No product found to delete",
                        success: false
                    }
                );
            }
        }
    })
}
