/*!
 * Module dependencies.
 */

var mongoose = require('mongoose')
var config = require('../../config/env/development')
var async = require('async')
var utils = require('../../utils')
var defaultUserConfig = require('../../config/default_profile.json')
var fs = require('fs')
const { resolveSoa } = require('dns')
var UserProfile = mongoose.model('UserProfile')
var UserFollower = mongoose.model('UserFollower')

exports.read = function (req, res) {
  UserProfile.find()
    .select('deviceId email')
    .limit(10)
    .sort({ id: -1 })
    .exec(function (err, userProfiles) {
      if (err) {
        return utils.sendFailureResponse(
          res,
          500,
          1011,
          err.message,
          'Failed to get user profiles',
          JSON.stringify(err.errors)
        )
      } else {
        return utils.sendSuccessResponse(res, 200, userProfiles)
      }
    })
}

exports.search = function (req, res) {
  var queryString = req.query.query
  var deviceId = req.deviceId
  var userId = ''
  async.waterfall(
    [
      function (callback) {
        UserProfile.findOne({ deviceId: deviceId }).exec(callback)
      },
      function (userprofile, callback) {
        userId = userprofile._id.toString()
        UserFollower.find({ userId: userId, isFollowing: true })
          .distinct('followerId')
          .lean()
          .exec(function (err, followers) {
            var followers = followers.map(function (follower) {
              return follower.toString()
            })
            callback(err, followers)
          })
      },
      function (followingUsers, callback) {
        UserProfile.find({
          $where: 'this._id.str.match(/' + queryString + '/)'
        })
          .select('coins xp level name')
          .exec(function (err, searchProfiles) {
            searchProfiles = searchProfiles.map(function (searchProfile) {
              let updatedProfile = {
                _id: searchProfile._id.toString(),
                xp: searchProfile.xp,
                coins: searchProfile.coins,
                level: searchProfile.level,
                name: searchProfile.name,
                isFollowing: false
              }
              if (followingUsers.indexOf(searchProfile._id.toString()) !== -1) {
                updatedProfile.isFollowing = true
              }
              return updatedProfile
            })
            searchProfiles = searchProfiles.filter(function (searchProfile) {
              if (searchProfile._id === userId) {
                return false
              } else {
                return true
              }
            })
            callback(err, searchProfiles)
          })
      }
    ],
    function (err, result) {
      if (err) {
        return utils.sendFailureResponse(
          res,
          500,
          1011,
          err.message,
          'Failed to unfollow user',
          JSON.stringify(err.errors)
        )
      } else {
        return utils.sendSuccessResponse(res, 200, result)
      }
    }
  )
}

exports.detail = function (req, res) {
  var deviceId = req.params.deviceId
  UserProfile.findOne({ deviceId: deviceId }).exec(function (err, userProfile) {
    if (err) {
      return utils.sendFailureResponse(
        res,
        500,
        1011,
        err.message,
        'Failed to get user profile detail',
        JSON.stringify(err.errors)
      )
    } else if (!userProfile) {
      var newUserProfile = new UserProfile(defaultUserConfig)
      return utils.sendSuccessResponse(res, 200, newUserProfile.toObject())
    } else {
      return utils.sendSuccessResponse(res, 200, userProfile)
    }
  })
}

exports.create = function (req, res) {
  var userProfile = new UserProfile(req.body)
  userProfile.save(function (err, savedUserProfile) {
    if (err) {
      return utils.sendFailureResponse(
        res,
        500,
        1011,
        err.message,
        'Failed to create a new user profile',
        JSON.stringify(err.errors)
      )
    } else {
      return utils.sendSuccessResponse(res, 200, savedUserProfile)
    }
  })
}

exports.update = function (req, res) {
  console.log("updateRequestBengins")
  console.log("req.file:"+req.file)
  console.log("req.file.path:"+req.file.path)
  console.log("req.body.deviceID:"+req.body.deviceId)
  async.waterfall(
    [
      function (callback) {
        fs.readFile(req.file.path, 'utf-8', callback)
      },
      function (fileData, callback) {
        var deviceId = req.body.deviceId
        var descyptedFileData = utils.decryptStringUsingXXtea(fileData)
        var updateParams = JSON.parse(descyptedFileData)
        UserProfile.findOneAndUpdate(
          { deviceId: deviceId },
          { $set: updateParams },
          { new: true, upsert: true }
        ).exec(callback)
      }
    ],
    function (err, updatedUserProfile) {
      if (err) {
        return utils.sendFailureResponse(
          res,
          500,
          1011,
          err.message,
          'Failed to update user profile',
          JSON.stringify(err.errors)
        )
      } else {
        return utils.sendSuccessResponse(res, 200, updatedUserProfile)
      }
    }
  )
}

exports.delete = function (req, res) {
  var profileId = req.params.profileId

  UserProfile.remove({ _id: profileId }).exec(function (err, data) {
    if (err) {
      return utils.sendFailureResponse(
        res,
        500,
        1011,
        err.message,
        'Failed to delete user profile',
        JSON.stringify(err.errors)
      )
    } else {
      if (data['result']['n'] == 1) {
        return utils.sendSuccessResponse(res, 200, {
          message: 'Deleted user profile'
        })
      } else {
        return utils.sendSuccessResponse(res, 200, {
          message: 'No user profile was  found to delete'
        })
      }
    }
  })
}

exports.getDefaultProfile = function (req, res) {
  fs.readFile(
    __dirname + '/../../config/default_profile.json',
    'utf-8',
    function (err, data) {
      if (err) {
        return utils.sendFailureResponse(
          res,
          500,
          1011,
          err.message,
          'Failed to get default user profile',
          JSON.stringify(err.errors)
        )
      }
      return utils.sendSuccessResponse(res, 200, JSON.parse(data))
    }
  )
}

exports.saveDefaultProfile = function (req, res) {
  var data = JSON.stringify(req.body)
  fs.writeFile(
    __dirname + '/../../config/default_profile.json',
    data,
    function (err) {
      if (err) {
        return utils.sendFailureResponse(
          res,
          500,
          1011,
          err.message,
          'Failed to update default user profile',
          JSON.stringify(err.errors)
        )
      }
      return utils.sendSuccessResponse(res, 200, {
        message: 'successfully update default profile'
      })
    }
  )
}

exports.getUsersByAppealCount = function (req, res) {
  console.log('RESULT  = ')
  UserProfile.aggregate([
    {
      $match: {}
    },
    {
      $project: {
        deviceId: '$deviceId',
        name: '$name',
        level: '$level',
        fbId: '$fbId',
        coins: '$coins',
        xp: '$xp',
        buildings: {
          $reduce: {
            input: '$buildings',
            initialValue: 0,
            in: { $add: ['$$value', '$$this.appeal'] }
          }
        }
      }
    },
    {
      $group: {
        _id: {
          deviceId: '$deviceId',
          name: '$name',
          level: '$level',
          fbId: '$fbId',
          coins: '$coins',
          xp: '$xp'
        },
        appealcount: { $sum: '$buildings' }
      }
    }
  ])
    .sort({ appealcount: -1 })
    .limit(100)
    .then(function (result) {
      // if (err) {
      //     console.log("ERROR OCCURRED. Error: ", err);
      //     return utils.sendFailureResponse(res, 500, 1011, err.message, 'Failed to Get User Profile', JSON.stringify(err.errors));
      // }
      // console.log("RESULT  = ", result);
      var finalResult = []
      for (var i = 0; i < result.length; i++) {
        finalResult.push({
          deviceId: result[i]._id.deviceId,
          name: result[i]._id.name,
          level: result[i]._id.level,
          fbId: result[i]._id.fbId,
          coins: result[i]._id.coins,
          xp: result[i]._id.xp,
          appealcount: result[i].appealcount
        })
      }
      if (finalResult) {
        return res.json({
          success: true,
          message: 'Found User Profile Data.',
          data: finalResult
        })
      } else {
        return res.json({
          success: true,
          message: 'User Profile data not found.',
          data: finalResult
        })
      }
    })
    .catch(function (err) {
      console.log('Error : ', err)
      return res.json({
        success: false,
        message: 'Failed to get User Appeal Count. Please try again later.'
      })
    })

  // UserProfile.aggregate([
  //     {
  //         $group: {
  //             _id: {
  //                 "deviceId": '$deviceId'
  //             },
  //             buildings: { $push: "$buildings" }
  //         }
  //     }, {
  //         $unwind: "$buildings"
  //     }, {
  //         $group: {
  //             _id: "$_id",
  //             appealcount: { $sum: "$buildings.appeal" }
  //         }
  //     },
  //     // { cursor: {} }
  // ], { cursor: {} })
  //     .then(result => {
  //         console.log("RESULT  = ", result);
  //         res.json({ "success": result })
  //     })
  // .catch(err => {
  //     res.json({ "error": err })
  // })
}

exports.getUsersByCoins = function (req, res) {
  UserProfile.aggregate([
    {
      $match: {}
    },
    {
      $project: {
        deviceId: '$deviceId',
        name: '$name',
        level: '$level',
        fbId: '$fbId',
        coins: '$coins',
        xp: '$xp',
        buildings: {
          $reduce: {
            input: '$buildings',
            initialValue: 0,
            in: { $add: ['$$value', '$$this.appeal'] }
          }
        }
      }
    },
    {
      $group: {
        _id: {
          deviceId: '$deviceId',
          name: '$name',
          level: '$level',
          fbId: '$fbId',
          coins: '$coins',
          xp: '$xp'
        },
        appealcount: { $sum: '$buildings' }
      }
    }
  ])
    .sort({ '_id.coins': -1 })
    .limit(100)
    .then(result => {
      console.log('RESULT  = ', result.length)
      var finalResult = []
      for (var i = 0; i < result.length; i++) {
        finalResult.push({
          deviceId: result[i]._id.deviceId,
          name: result[i]._id.name,
          level: result[i]._id.level,
          fbId: result[i]._id.fbId,
          coins: result[i]._id.coins,
          xp: result[i]._id.xp,
          appealcount: result[i].appealcount
        })
      }
      if (finalResult) {
        res.json({
          success: true,
          message: 'Found User Data.',
          data: finalResult
        })
      } else {
        res.json({
          success: false,
          message: 'No User Profile found.',
          data: finalResult
        })
      }
    })
    .catch(error => {
      res.json({
        success: false,
        message: 'Failed to get User Profile data. Please try again later.'
      })
    })
}

exports.getUserslevelnumber = function (req, res) {
  console.log('we are here')
  UserProfile.aggregate([
    {
      $match: {}
    },
    {
      $project: {
        deviceId: '$deviceId',
        name: '$name',
        level: '$level',
        fbId: '$fbId',
        coins: '$coins',
        xp: '$xp',
        buildings: {
          $reduce: {
            input: '$buildings',
            initialValue: 0,
            in: { $add: ['$$value', '$$this.appeal'] }
          }
        }
      }
    },
    {
      $group: {
        _id: {
          deviceId: '$deviceId',
          name: '$name',
          level: '$level',
          fbId: '$fbId',
          coins: '$coins',
          xp: '$xp'
        },
        appealcount: { $sum: '$buildings' }
      }
    }
  ])
    .sort({ '_id.level': -1 })
    .then(result => {
      // console.log("RESULT  = ", result);
      var finalResult = []
      for (var i = 0; i < result.length; i++) {
        finalResult.push({
          deviceId: result[i]._id.deviceId,
          name: result[i]._id.name,
          level: result[i]._id.level,
          fbId: result[i]._id.fbId,
          coins: result[i]._id.coins,
          xp: result[i]._id.xp,
          appealcount: result[i].appealcount
        })
      }
      if (finalResult) {
        res.json({
          success: true,
          message: 'Found User Data.',
          data: finalResult
        })
      } else {
        res.json({
          success: false,
          message: 'No User Profile found.',
          data: finalResult
        })
      }
    })
    .catch(error => {
      res.json({
        success: false,
        message: 'Failed to get User Profile data. Please try again later.'
      })
    })
}
exports.getUsers = function (req, res) {
  var search = {}
  if (req.query.deviceFilter) {
    search['device'] = req.query.deviceFilter
  }
  if (req.query.userFilter) {
    if (req.query.userFilter == 'facebook') {
      search['fbId'] = { $ne: null }
    } else {
      search['fbId'] = { $eq: null }
    }
    // search.userFilter = req.query.userFilter;
  }
  if (req.query.searchOn) {
    search['$or'] = []

    const obj = new Object()
    const columnName = req.query.searchOn
    console.log(columnName)

    obj[columnName] = new RegExp(req.query.search, 'i')
    search['$or'].push(obj)
  }
  console.log('SEARCH = ', search)
  UserProfile.find(search)
    .sort([['_id', -1]])
    .skip(parseInt(req.query.start))
    .limit(parseInt(req.query.length))
    .then(function (result) {
      UserProfile.aggregate(
        [{ $match: search }, { $count: 'totalCount' }],
        function (err, rescount) {
          var response = new Object()
          response.draw = req.query.draw
          response.recordsTotal =
            rescount.length > 0 ? rescount[0].totalCount : 0
          response.recordsFiltered =
            rescount.length > 0 ? rescount[0].totalCount : 0
          response.data = result
          res.json(response)
        }
      )
    })
    .catch(function (err) {
      // res.json({
      //     success: false,
      //     message: "Failed to get User Profile. Please try again later."
      // })
      const response = new Object()
      response.draw = req.query.draw
      response.recordsTotal = 0
      response.recordsFiltered = 0
      response.data = []
      res.json(response)
    })
}

exports.deleteUserProfile = function (req, res) {
  UserProfile.deleteOne({ _id: req.query._id })
    .then(function (result) {
      if (result) {
        res.json({
          success: true,
          message: 'User Profile Deleted Successfully'
        })
      } else {
        res.json({
          success: false,
          message:
            'Failed to delete User Profile. No Profile found with this id'
        })
      }
    })
    .catch(function (err) {
      res.json({
        success: false,
        message: 'Failed to delete User Profile. Please try again later.'
      })
    })
}

exports.updateUser = function (req, res) {
  var setObj = {}
  var objUser = req.body
  if (objUser.deviceId) {
    setObj.deviceId = objUser.deviceId
  }
  if (objUser.level) {
    setObj.level = objUser.level
  }
  if (objUser.xp) {
    setObj.xp = objUser.xp
  }
  if (objUser.coins) {
    setObj.coins = objUser.coins
  }
  if (objUser.pushToken) {
    setObj.pushToken = objUser.pushToken
  }
  if (objUser.device) {
    setObj.device = objUser.device
  }
  if (objUser.fbId) {
    setObj.fbId = objUser.fbId
  }
  if (objUser.name) {
    setObj.name = objUser.name
  }
  if (objUser.diamonds) {
    setObj.diamonds = objUser.diamonds
  }
  console.log('SET OBJ = ', setObj)
  UserProfile.updateOne(
    {
      _id: req.body._id
    },
    {
      $set: setObj
    }
  )
    .then(function (result) {
      if (result) {
        res.json({
          success: true,
          message: 'User Profile updated successfully.',
          data: result
        })
      } else {
        res.json({
          success: false,
          message: 'Failed to update User Profile.'
        })
      }
    })
    .catch(function (err) {
      res.json({
        success: false,
        message: 'Failed to update User Profile. Please try again later.'
      })
    })
}
