/*!
* Module dependencies
*/

var mongoose = require('mongoose');
var Schema = mongoose.Schema;

var UpgradeInfoSchema = new Schema({
    level: {
        type: Number, //level of the building
    },
    maxCoins: {
        type: Number, //max number of coins to be accumalated for that level
    },
    initialCoins: {
        type: Number, //Not required can be used to set initial coins
        default: 0
    },
    usageFee: {
        type: Number // fees to use the building
    },
    cost: {
        type: Number, // cost to upgrade
    },
    speedUpCost: {
        type: Number, //speed up cost using diamonds
    },
    appeal: {
        type: Number, //field to determine how popular is the shop
    },
    time: {
        type: Number //time required to upgrade the shop
    },
    coinRate: {
        type: Number, // passive coin generations - coins per second in background
        default: 0
    }
})

var BuildingSchema = new Schema({
    name: {
        type: String,
        required: true
    },
    type: {
        type: String,
        enum: ['shop', 'decorative', 'utility', 'machine' ],
        required: true
    },
    subType: {
        type: Number,
        required: true
    },
    unlockLevel: {
        type: Number,
        required: true
    },
    imageUrl: {
        type: String,
        required: true
    },
    imageInfo: {
        name: {
            type: String,
            required: true
        },
        type: {
            type: String,
            enum: ['normal', 'custom']
        },
        file: {
            type: String,
            default: "Components/building_img.csb"
        }
    },
    enabled: {
        type: Boolean,
        default: true
    },
    dialogues: [{
        type: String
    }],
    translation: {
        type: Object
    },
    upgradeInfo : [UpgradeInfoSchema]
});

/**
* Register
*/

mongoose.model('Building', BuildingSchema);
